@@ -1,34 +1,9 @@
 import argparse
 import json
+
 from explainaboard import get_loader, get_processor
 from explainaboard import TaskType
-
-
-def get_performance_gap(sys1, sys2):
-
-    for metric_name, performance_unit in sys1["results"]["overall"].items():
-        sys1["results"]["overall"][metric_name]["value"] = float(
-            sys1["results"]["overall"][metric_name]["value"]
-        ) - float(sys2["results"]["overall"][metric_name]["value"])
-        sys1["results"]["overall"][metric_name]["confidence_score_low"] = 0
-        sys1["results"]["overall"][metric_name]["confidence_score_up"] = 0
-
-    for attr, performance_list in sys1["results"]["fine_grained"].items():
-        for idx, performances in enumerate(performance_list):
-            for idy, performance_unit in enumerate(
-                performances
-            ):  # multiple metrics' results
-                sys1["results"]["fine_grained"][attr][idx][idy]["value"] = float(
-                    sys1["results"]["fine_grained"][attr][idx][idy]["value"]
-                ) - float(sys2["results"]["fine_grained"][attr][idx][idy]["value"])
-                sys1["results"]["fine_grained"][attr][idx][idy][
-                    "confidence_score_low"
-                ] = 0
-                sys1["results"]["fine_grained"][attr][idx][idy][
-                    "confidence_score_up"
-                ] = 0
-
-    return sys1
+from explainaboard import get_pairwise_performance_gap
 
 
 def main():
@@ -61,6 +36,22 @@ def main():
         help="the name of dataset",
     )
 
+    parser.add_argument(
+        '--sub_dataset',
+        type=str,
+        required=False,
+        default=None,
+        help="the name of sub-dataset",
+    )
+
+    parser.add_argument(
+        '--reload_stat',
+        type=str,
+        required=False,
+        default=None,
+        help="reload precomputed statistics over training set (if exists)",
+    )
+
     parser.add_argument(
         '--metrics',
         type=str,
@@ -72,7 +63,9 @@ def main():
     args = parser.parse_args()
 
     dataset = args.dataset
+    sub_dataset = args.sub_dataset
     task = args.task
+    reload_stat = False if args.reload_stat == "0" else True
     system_outputs = args.system_outputs
     num_outputs = len(system_outputs)
     metric_names = args.metrics
@@ -89,6 +82,11 @@ def main():
 
     # Read in data and check validity
     system_datasets = [get_loader(task, data=x).load() for x in system_outputs]
+
+    # Get user_defined_features_configs (this should be generalized later
+    loader = get_loader(task, data=system_outputs[0])
+    user_defined_features_configs = loader.load_user_defined_features_configs()
+
     if len(system_datasets) == 2 and len(system_datasets[0]) != len(system_datasets[1]):
         num0 = len(system_datasets[0])
         num1 = len(system_datasets[1])
@@ -97,16 +95,27 @@ def main():
         )
 
     # Setup metadata
-    metadata = {"dataset_name": dataset, "task_name": task}
+    metadata = {
+        "dataset_name": dataset,
+        "sub_dataset_name": sub_dataset,
+        "task_name": task,
+        "reload_stat": reload_stat,
+        "user_defined_features_configs": user_defined_features_configs,
+    }
     if metric_names is not None:
         metadata["metric_names"] = metric_names
 
     # Run analysis
-    reports = [get_processor(task, metadata=metadata, data=x).process() for x in system_datasets]
+    reports = [
+        get_processor(task, metadata=metadata, data=x).process()
+        for x in system_datasets
+    ]
     if len(system_outputs) == 1:  # individual system analysis
         reports[0].print_as_json()
-    else:                         # pairwise analysis
-        compare_analysis = get_performance_gap(reports[0].to_dict(), reports[1].to_dict())
+    else:  # pairwise analysis
+        compare_analysis = get_pairwise_performance_gap(
+            reports[0].to_dict(), reports[1].to_dict()
+        )
         print(json.dumps(compare_analysis, indent=4))
 
 
