@@ -63,7 +63,7 @@ class Result:
     calibration: List[Performance] = None
     fine_grained: Any = None
     is_print_case: bool = True
-    is_print_confidence_interval: bool = True
+    is_print_confidence_interval: bool = False
 
 
 @dataclass
@@ -85,7 +85,9 @@ class SysOutputInfo:
     task_name: str
     model_name: Optional[str] = None
     dataset_name: Optional[str] = None
+    sub_dataset_name: Optional[str] = None
     metric_names: Optional[List[str]] = None
+    reload_stat: bool = True
     # language : str = "English"
 
     # set later
