@@ -3,7 +3,7 @@ from .loader import get_loader
 
 from . import text_classification
 from . import named_entity_recognition
-from . import extractive_qa_squad
+from . import extractive_qa
 from . import conditional_generation
 from . import text_pair_classification
 from . import hellaswag
