@@ -18,6 +18,9 @@ class Loader:
         self._file_type = file_type
         self._data = data
 
+    def load_user_defined_features_configs(self):
+        return {}
+
     def _load_raw_data_points(self) -> Iterable:
         """
         loads data and return an iterable of data points. element type depends on file_type
@@ -31,6 +34,8 @@ class Loader:
                 return self._data.splitlines()
             elif self._file_type == FileType.json:
                 return json.loads(self._data)
+            elif self._file_type == FileType.datalab:
+                return self._data
             else:
                 raise NotImplementedError
 
