@@ -1,11 +1,11 @@
 from typing import Dict, Iterable, List
-from explainaboard.constants import *
+from explainaboard.constants import Source, FileType
+from enum import Enum
 from explainaboard.tasks import TaskType
 from .loader import register_loader
 from .loader import Loader
 
 
-
 @register_loader(TaskType.text_pair_classification)
 class TextPairClassificationLoader(Loader):
     """
@@ -16,19 +16,17 @@ class TextPairClassificationLoader(Loader):
         please refer to `test_loaders.py`
     """
 
-    def __init__(self, source: Source, file_type: Enum, data :str = None):
+    def __init__(self, source: Source, file_type: Enum, data: str = None):
 
-        if source == None:
+        if source is None:
             source = Source.local_filesystem
-        if file_type == None:
+        if file_type is None:
             file_type = FileType.tsv
 
-
         self._source = source
         self._file_type = file_type
         self._data = data
 
-
     def load(self) -> Iterable[Dict]:
         """
         :param path_system_output: the path of system output file with following format:
@@ -40,11 +38,15 @@ class TextPairClassificationLoader(Loader):
         if self._file_type == FileType.tsv:
             for id, dp in enumerate(raw_data):
                 text1, text2, true_label, predicted_label = dp[:4]
-                data.append({"id": str(id),
-                             "text1": text1.strip(),
-                             "text2": text2.strip(),
-                             "true_label": true_label.strip(),
-                             "predicted_label": predicted_label.strip()})
+                data.append(
+                    {
+                        "id": str(id),
+                        "text1": text1.strip(),
+                        "text2": text2.strip(),
+                        "true_label": true_label.strip(),
+                        "predicted_label": predicted_label.strip(),
+                    }
+                )
         else:
             raise NotImplementedError
-        return data
\ No newline at end of file
+        return data
