@@ -2,7 +2,7 @@
 # when a new processor is implemented, remember to import it here so it gets registered
 from . import text_classification
 from . import named_entity_recognition
-from . import extractive_qa_squad
+from . import extractive_qa
 from . import conditional_generation
 
 from . import text_pair_classification
