@@ -15,8 +15,6 @@ class KGLinkTailPredictionProcessor(Processor):
             "link": feature.Value("string"),
             "true_tail": feature.Value("string"),
             "predicted_tails": feature.Sequence(feature.Value("string")),
-            # ============================================
-            # START WARM-UP TASKS
             "tail_entity_length": feature.Value(
                 dtype="float",
                 description="number of words in the tail entity",
@@ -46,6 +44,7 @@ class KGLinkTailPredictionProcessor(Processor):
                     _number=4,
                     _setting=(),
                 ),
+                require_training_set=True,
             ),
             "link_fre": feature.Value(
                 dtype="float",
@@ -56,6 +55,7 @@ class KGLinkTailPredictionProcessor(Processor):
                     _number=4,
                     _setting=(),
                 ),
+                require_training_set=True,
             ),
             "head_fre": feature.Value(
                 dtype="float",
@@ -66,6 +66,23 @@ class KGLinkTailPredictionProcessor(Processor):
                     _number=4,
                     _setting=(),
                 ),
+                require_training_set=True,
+            ),
+            "symmetry": feature.Value(
+                dtype="string",
+                description="boolean feature: 'symmetric' or 'asymmetric'; more granularity to be added",
+                is_bucket=True,
+                bucket_info=feature.BucketInfo(
+                    _method="bucket_attribute_discrete_value", _number=2, _setting=1
+                ),
+            ),
+            "entity_type_level": feature.Value(
+                dtype="string",
+                description="most specific (highest) entity type level of true tail entity",
+                is_bucket=True,
+                bucket_info=feature.BucketInfo(
+                    _method="bucket_attribute_discrete_value", _number=8, _setting=1
+                ),
             ),
         }
     )
@@ -76,8 +93,12 @@ class KGLinkTailPredictionProcessor(Processor):
         if "task_name" not in metadata.keys():
             metadata["task_name"] = TaskType.kg_link_tail_prediction.value
         if "metric_names" not in metadata.keys():
-            metadata["metric_names"] = ["Hits"]
+            metadata["metric_names"] = ["Hits", "MeanReciprocalRank"]
         super().__init__(metadata, system_output_data)
         self._builder = KGLTPExplainaboardBuilder(
-            self._system_output_info, system_output_data
+            self._system_output_info,
+            system_output_data,
+            user_defined_feature_config=metadata.get(
+                "user_defined_features_configs", None
+            ),
         )
