@@ -52,6 +52,28 @@ class NERProcessor(Processor):
                     _setting=(),
                 ),
             ),
+            "num_oov": feature.Value(
+                dtype="float",
+                description="the number of out-of-vocabulary words",
+                is_bucket=True,
+                bucket_info=feature.BucketInfo(
+                    _method="bucket_attribute_specified_bucket_value",
+                    _number=4,
+                    _setting=(),
+                ),
+                require_training_set=True,
+            ),
+            "fre_rank": feature.Value(
+                dtype="float",
+                description="the average rank of each work based on its frequency in training set",
+                is_bucket=True,
+                bucket_info=feature.BucketInfo(
+                    _method="bucket_attribute_specified_bucket_value",
+                    _number=4,
+                    _setting=(),
+                ),
+                require_training_set=True,
+            ),
             "true_entity_info": feature.Sequence(
                 feature.Set(
                     {
@@ -128,7 +150,7 @@ class NERProcessor(Processor):
                             dtype="float",
                             description="entity label consistency",
                             is_bucket=True,
-                            is_pre_computed=True,
+                            require_training_set=True,
                             bucket_info=feature.BucketInfo(
                                 _method="bucket_attribute_specified_bucket_value",
                                 _number=4,
@@ -139,7 +161,7 @@ class NERProcessor(Processor):
                             dtype="float",
                             description="entity frequency",
                             is_bucket=True,
-                            is_pre_computed=True,
+                            require_training_set=True,
                             bucket_info=feature.BucketInfo(
                                 _method="bucket_attribute_specified_bucket_value",
                                 _number=4,
