@@ -48,8 +48,6 @@ table_schemas[TaskType.text_classification] = [
 ]
 
 
-
-
 table_schemas[TaskType.text_pair_classification] = [
     {"field_key": "text1", "sort": False, "filter": False, "label": "Text1"},
     {"field_key": "text2", "sort": False, "filter": False, "label": "Text2"},
@@ -92,7 +90,7 @@ table_schemas[TaskType.summarization] = [
     },
 ]
 
-table_schemas[TaskType.extractive_qa_squad] = [
+table_schemas[TaskType.question_answering_extractive] = [
     {"field_key": "context", "sort": False, "filter": True, "label": "Context"},
     {"field_key": "question", "sort": False, "filter": True, "label": "Question"},
     {
